------------------------------------------------------------------------------
--  This file is a part of the LPP VHDL IP LIBRARY
--  Copyright (C) 2009 - 2010, Laboratory of Plasmas Physic - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
-------------------------------------------------------------------------------
--                    Author : Paul Leroy
--                     Mail : paul.leroy@lpp.polytechnique.fr
----------------------------------------------------------------------------


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

PACKAGE PLE_lpp_fft IS

COMPONENT BUTTERFLY_DATAFLOW
  GENERIC (
    Sample_SZ    : INTEGER := 16);
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    Are            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Aim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bre            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    c_in           : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cps_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cms_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    op1            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    op2            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    out_alu        : OUT STD_LOGIC_VECTOR ( 2*Sample_SZ-1 DOWNTO 0);

    sel_op1        : IN STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Are Aim X Y Z
    sel_op2        : IN STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Bre Bim C_in cps_in cms_in
    sel_xyz        : IN STD_LOGIC_VECTOR( 2 DOWNTO 0 ); -- X Y Z
    alu_ctrl       : IN STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : IN STD_LOGIC_VECTOR( 1 DOWNTO 0 )
    );
END COMPONENT;

COMPONENT BUTTERFLY_CTRL
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    sample_in_val  : IN  STD_LOGIC;
    sample_out_val : OUT STD_LOGIC;

    sel_op1        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Are Aim X Y Z
    sel_op2        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 ); -- Bre Bim C_in cps_in cms_in
    sel_xyz        : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 ); -- X Y Z
    sel_out        : OUT STD_LOGIC_VECTOR( 4 DOWNTO 0 );
    alu_ctrl       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : OUT STD_LOGIC_VECTOR( 1 DOWNTO 0 )
    );
END COMPONENT;

COMPONENT BUTTERFLY_TOP
  GENERIC (
    Sample_SZ    : INTEGER := 16);
  PORT (
    rstn : IN STD_LOGIC;
    clk  : IN STD_LOGIC;

    sample_in_val  : IN  STD_LOGIC;
    sample_out_val : OUT STD_LOGIC;

    Are            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Aim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bre            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    Bim            : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    c_in           : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cps_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    cms_in         : IN STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);

    op1            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    op2            : OUT STD_LOGIC_VECTOR ( Sample_SZ-1 DOWNTO 0);
    alu_ctrl       : OUT STD_LOGIC_VECTOR( 2 DOWNTO 0 );
    alu_comp       : OUT STD_LOGIC_VECTOR( 1 DOWNTO 0 );

    butterfly_out  : OUT STD_LOGIC_VECTOR ( 2*Sample_SZ-1 DOWNTO 0);
    sel_out        : OUT STD_LOGIC_VECTOR ( 4 DOWNTO 0)
    );
END COMPONENT;

END;